/********************************************************************************/
/*										*/
/*		bouncecontext.cxx						*/
/*										*/
/*	Drawing context for bouncing balls demo program 			*/
/*										*/
/********************************************************************************/


#include "bounce_local.H"

#include "bounce_context.H"
#include "bounce_view.H"



/********************************************************************************/
/*										*/
/*	Color Tables								*/
/*										*/
/********************************************************************************/


static	COLORREF color_values[] = {
   0x000000, 0x0000ff, 0x00ff00, 0x00ffff, 0xff0000, 0xff00ff, 0xffff00
};



/********************************************************************************/
/*										*/
/*	BounceDrawContextInfo constuructors/destructors 			*/
/*										*/
/********************************************************************************/


BounceDrawContextInfo::BounceDrawContextInfo()
{
   cur_color = BOUNCE_COLOR_NONE;
   line_style = BOUNCE_LINE_UNDEF;
   cur_shape = BOUNCE_SHAPE_SQUARE;

   device_context = NULL;
   cur_pen = NULL;
   pen_brush = NULL;

   setupColors();
}



BounceDrawContextInfo::~BounceDrawContextInfo()
{
   Integer i;

   for (i = 0; i < BOUNCE_NUM_COLOR; ++i) delete color_table[i];

   delete pen_brush;
   delete cur_pen;

   delete device_context;
}



/********************************************************************************/
/*										*/
/*	BounceDrawContextInfo setup methods					*/
/*										*/
/********************************************************************************/


void
BounceDrawContextInfo::setCDC(BounceView bv)
{
   device_context = new CClientDC(bv);

   device_context->SetMapMode(MM_TEXT);
   device_context->SetViewportOrg(0,0);
   device_context->SetBkColor(BACK_COLOR^BACK_COLOR);
   device_context->SetBkMode(OPAQUE);
   device_context->SetROP2(R2_XORPEN);
}



/********************************************************************************/
/*										*/
/*	BounceDrawContextInfo clearing methods					*/
/*										*/
/********************************************************************************/


void
BounceDrawContextInfo::clear()
{
   RECT wbnds;

   if (device_context == NULL) return;

   device_context->GetWindow()->GetClientRect(&wbnds);

   device_context->SetROP2(R2_COPYPEN);
   device_context->FillSolidRect(&wbnds,BACK_COLOR);
   device_context->SetROP2(R2_XORPEN);
}





/********************************************************************************/
/*										*/
/*	Color Management methods						*/
/*										*/
/********************************************************************************/


void
BounceDrawContextInfo::setupColors()
{
   COLORREF r;

   for (Integer i = 0; i < BOUNCE_NUM_COLOR; ++i) {
      r = color_values[i] ^ BACK_COLOR;
      color_table[i] = new CBrush(r);
    }

   pen_brush = new CBrush(LINE_COLOR^BACK_COLOR);
}



/********************************************************************************/
/*										*/
/*	BounceDrawContextInfo property setting methods				*/
/*										*/
/********************************************************************************/


void
BounceDrawContextInfo::setLineStyle(BounceLine ls)
{
   CPen * newpen;
   COLORREF lcol;
   DWORD dots[10];

   if (ls == line_style) return;

   line_style = ls;
   lcol = LINE_COLOR ^ BACK_COLOR;
   dots[0] = dots[1] = 3;
   LOGBRUSH lbr;
   pen_brush->GetLogBrush(&lbr);

   switch (line_style) {
      case BOUNCE_LINE_NONE :
	 newpen = new CPen(PS_NULL,1,lcol);
	 break;
      case BOUNCE_LINE_SOLID :
	 newpen = new CPen(PS_SOLID,1,lcol);
	 break;
      case BOUNCE_LINE_THICK_SOLID :
	 newpen = new CPen(PS_SOLID,3,lcol);
	 break;
      case BOUNCE_LINE_DOTTED :
	 newpen = new CPen(PS_GEOMETRIC|PS_USERSTYLE,1,&lbr,2,dots);
	 break;
      case BOUNCE_LINE_THICK_DOTTED :
	 newpen = new CPen(PS_GEOMETRIC|PS_USERSTYLE,3,&lbr,2,dots);
	 break;
    }

   device_context->SelectObject(newpen);
   delete cur_pen;
   cur_pen = newpen;
}



void
BounceDrawContextInfo::setColor(BounceColor c)
{
   Integer i;

   if (cur_color == c) return;

   cur_color = c;

   for (i = 0; i < BOUNCE_NUM_COLOR; ++i) {
      if (((1 << i) & cur_color) != 0) break;
    }
   if (i >= BOUNCE_NUM_COLOR) return;

   device_context->SelectObject(color_table[i]);
}



void
BounceDrawContextInfo::setShape(BounceShape s)
{
   cur_shape = s;
}



/********************************************************************************/
/*										*/
/*	BounceDrawContextInfo drawing methods					*/
/*										*/
/********************************************************************************/


void
BounceDrawContextInfo::draw(Integer x,Integer y,Integer r)
{
   Integer lx,ty,wd,ht;

   wd = ht = (Integer) r * 2;
   lx = (Integer) (x - r);
   ty = (Integer) (y - r);

   switch (cur_shape) {
      case BOUNCE_SHAPE_CIRCLE :
	 drawCircle(lx,ty,lx+wd,ty+ht);
	 break;
      case BOUNCE_SHAPE_SQUARE :
	 drawSquare(lx,ty,lx+wd,ty+ht);
	 break;
      case BOUNCE_SHAPE_DIAMOND :
	 drawDiamond(lx,ty,wd,ht);
	 break;
    }
}



void
BounceDrawContextInfo::drawCircle(Integer lx,Integer ty,Integer rx,Integer by)
{
   device_context->Ellipse(lx,ty,rx,by);
}




void
BounceDrawContextInfo::drawDiamond(Integer lx,Integer ty,Integer wd,Integer ht)
{
   CPoint pts[5];

   pts[0].x = pts[2].x = lx+wd/2;
   pts[0].y = ty;
   pts[1].x = lx+wd;
   pts[1].y = pts[3].y = ty+ht/2;
   pts[2].y = ty+ht;
   pts[3].x = lx;
   pts[4] = pts[0];

   device_context->Polygon(pts,4);
}




void
BounceDrawContextInfo::drawSquare(Integer lx,Integer ty,Integer rx,Integer by)
{
   device_context->Rectangle(lx,ty,rx,by);
}




/* end of bouncecontext.cxx */
